/**********************************************************/
/*                                                        */
/*   File Name    :  DATADICT.C                           */
/*                                                        */
/*   Component    :  Database Dictionary Listing          */
/*                                                        */
/*   Product      :  DataPlus 'C' Library                 */
/*                                                        */
/*   Author       :  Andy Dransfield                      */
/*                                                        */
/*   Version history                                      */
/*                                                        */
/*   01 : January   1992                                  */
/*                                                        */
/**********************************************************/
/*                                                        */
/*   This program has been created by Andy Dransfield.    */
/*   It may not be copied (in whole or in part) without   */
/*   the prior written permission of the Author.          */
/*                                                        */
/**********************************************************/

#include <stdio.h>
#include <conio.h>
#include <string.h>

#include "DataPlus.h"

#define forever for (;;)

void print_form_details (char *);
void print_string (char *);

void main (int argc, char **argv)
{
    char directory_string [83],
        *directory,
         database_string  [22],
        *database,
         user_string      [18],
        *user,
         password_string  [11],
        *password,
         form_name        [21];

    int result,
        DataEase_version,
        form_number,
        max_forms;

    printf ("          ͻ\n");
    printf ("                                                               \n");
    printf ("            This utility has been developed by Hill Croft      \n");
    printf ("            Information Technologies to print a listing of     \n");
    printf ("            all the vital information required when            \n");
    printf ("            programming using the DataPlus C Library.          \n");
    printf ("                                                               \n");
    printf ("          ͼ\n");

    if (argc < 5)
    {
        printf ("Not enough parameters entered.  Use:\n\n");

        printf ("datadict <path> <database> <user> <password> [<form>]\n\n");
    }

    if (argc < 2)
    {
        printf ("Directory Path Name: ");

        directory_string [0] = 81;

        directory = cgets (directory_string);

        printf ("\n");
    }
    else
        directory = argv [1];

    if (argc < 3)
    {
        printf ("Database Name or Letter: ");

        database_string [0] = 20;

        database = cgets (database_string);

        printf ("\n");
    }
    else
        database = argv [2];

    if (argc < 4)
    {
        printf ("What is your name?: ");

        user_string [0] = 16;

        user = cgets (user_string);

        printf ("\n");
    }
    else
        user = argv [3];

    if (argc < 5)
    {
        printf ("What is your security password?: ");

        password_string [0] = 9;

        password = cgets (password_string);

        printf ("\n");
    }
    else
        password = argv [4];

    result = DP_open_database (directory, database [0]);

    if (result < 0)
    {
        printf ("DataPlus cannot open %c in %s (%d)\n",
                 database [0], directory, result);

        exit (-1);
    }

    result = DP_user_sign_on (user, password);

    if (result != 1)
    {
        printf ("Only the Database Administrator may list the Data Dictionary: %d\n", result);

        exit (-2);
    }

    DataEase_version = DP_DataEase_version ();

    DP_database_name (form_name);

    fprintf (stdprn, "DataPlus Data Dictionary Listing of '%.19s' in %s\r\n\n",
             form_name, directory);

    fprintf (stdprn, "DataEase Version: %d\r\n\n", DataEase_version);

    max_forms = DP_maximum_forms ();

    fprintf (stdprn, "FORM                 Existing Deleted Fields\r");
    fprintf (stdprn, "____________________   ____    ____     __\r\n\n");

    if (argc > 5)
    {
        print_form_details (argv [5]);
    }
    else
    for (form_number = 1; form_number < max_forms; form_number++)
    {
        DP_form_name (form_number, form_name);

        print_form_details (form_name);
    }

    fprintf (stdprn, "\f");

    fclose (stdprn);

    result = DP_first_error ();

    if (result)
        printf ("Fatal Error: %s\n", DP_error_message (result));

    printf ("Hit any key to continue ...");

    getch ();

    printf ("\n");

    printf ("     ͻ\n");
    printf ("                                                                         \n");
    printf ("       Hill Croft Information Technologies trust that you have found     \n");
    printf ("       this utility helpful.  This utility, and others, have been        \n");
    printf ("       developed using the DataPlus C Library.                           \n");
    printf ("                                                                         \n");
    printf ("       The DataPlus C Library provides developers using Borland C++,     \n");
    printf ("       Turbo C, Turbo C++, or the Microsoft C language compilers with    \n");
    printf ("       an Application Programming Interface of more than 75 functions    \n");
    printf ("       that allow access to data in forms of a DataEase database.        \n");
    printf ("                                                                         \n");
    printf ("       DataPlus comes in the form of a C library that can be linked with \n");
    printf ("       any programs you create.  Since any resultant application is a    \n");
    printf ("       standard executable file (.EXE), you can expect your programs to  \n");
    printf ("       realise the size and speed benefits normally associated with C.   \n");
    printf ("                                                                         \n");
    printf ("       The DataPlus C Library may be purchased for 450, and the full    \n");
    printf ("       range of utilities may be purchased for 25 from:                 \n");
    printf ("                                                                         \n");
    printf ("                   Hill Croft Information Technologies                   \n");
    printf ("             56 London Road, Loughton, Milton Keynes, England            \n");
    printf ("                         Telephone: (908) 666244                         \n");
    printf ("                                                                         \n");
    printf ("     ͼ");
}

void print_form_details (form_name)
char *form_name;
{
    char field_name       [21],
         *choice_name,
         field_type,
         field_char,
         *derivation,
         *lower_limit,
         *upper_limit,
         *help;

    int field_number,
        choice_number,
        field_size,
        choice_size,
        max_fields,
        max_choices,
        result;

    result = DP_load_form_attributes (form_name);

    if (result < 0)
    {
        printf ("DataPlus cannot open form %s (%d)\n", form_name, result);

        exit (-3);
    }

    max_fields = DP_maximum_fields (form_name);

    fprintf (stdprn, "%-20s   %4ld    %4ld     %2d\r\n",
             form_name,
             DP_existing_records (form_name),
             DP_deleted_records (form_name),
             max_fields);

    DP_form_help (form_name, &help);

    if (help)
    {
        fprintf (stdprn, "      Form Help:\r\n");

        print_string (help);
    }

    if (max_fields)
    {
        fprintf (stdprn, "\r\n");
        fprintf (stdprn, "No. Name                 Required Index Unique Virtual Type\r");
        fprintf (stdprn, "___ ____________________    ___    ___    ___    ___   _________\r\n\n");
    }

    for (field_number = 0; field_number < max_fields; field_number++)
    {
        DP_field_name (form_name, field_number, field_name);

        field_char = DP_field_attributes (form_name, field_name);

        fprintf (stdprn, "%3d %-20s ",
                 field_number + 1,
                 field_name);

        if (field_char & 0x01)
            fprintf (stdprn, "   yes   ");
        else
            fprintf (stdprn, "   no    ");

        if (field_char & 0x02)
            fprintf (stdprn, " yes  ");
        else
            fprintf (stdprn, " no   ");

        if (field_char & 0x08)
            fprintf (stdprn, "  yes  ");
        else
            fprintf (stdprn, "  no   ");

        if (field_char & 0x80)
        {
            fprintf (stdprn, "  yes\r\n");

            continue;
        }
        else
            fprintf (stdprn, "  no    ");

        field_type = DP_field_data_type (form_name, field_name);
        field_size = DP_field_size (form_name, field_name);

        if (field_type == 'C')
            fprintf (stdprn, "char [%d] ", field_size);
        else
        if (field_type == 'S')
            fprintf (stdprn, "signed char");
        else
        if (field_type == 'I')
            fprintf (stdprn, "int");
        else
        if (field_type == 'L')
            fprintf (stdprn, "long");
        else
        if (field_type == 'D')
            fprintf (stdprn, "double");
        else
        if (field_type == 'F')
            fprintf (stdprn, "float");
        else
        if (field_type == 'U')
            fprintf (stdprn, "char [%d] (%d Choices)",
                     DP_field_choice_size (form_name, field_name),
                     DP_maximum_field_choices (form_name, field_name));

        fprintf (stdprn, "\r\n");

        if (field_type == 'U')
        {
            max_choices = DP_maximum_field_choices (form_name, field_name);

            choice_size = DP_field_choice_size (form_name, field_name);

            for (choice_number = 1; choice_number <= max_choices; choice_number++)
            {
                DP_field_choice_name (form_name,
                                      field_name,
                                      choice_number,
                                      &choice_name);

                fprintf (stdprn, "    %d: %-*.*s\r\n",
                         choice_number, choice_size, choice_size, choice_name);
            }
        }

        if (field_char & 0x10)
        {
            DP_field_range (form_name, field_name, &lower_limit,
                                                   &upper_limit);

            if (lower_limit)
                fprintf (stdprn, "      Lower Limit: %s\r\n", lower_limit);

            if (upper_limit)
                fprintf (stdprn, "      Upper Limit: %s\r\n", upper_limit);
        }

        if (field_char & 0x20)
        {
            DP_field_derivation (form_name, field_name, &derivation);

            if (derivation)
            {
                fprintf (stdprn, "      Field Derivation:\r\n");

                print_string (derivation);
            }
        }

        DP_field_help (form_name, field_name, &help);

        if (help)
        {
            fprintf (stdprn, "      Field Help:\r\n");

            print_string (help);
        }
    }

    fprintf (stdprn, "\r\n");

    DP_clear_form_attributes (form_name);
}

void print_string (string)
char *string;
{
    forever
    {
        fprintf (stdprn, "%.*s\r\n", strcspn (string, "\r"), string);

        string = strchr (string, '\r');

        if (string == NULL)
            break;

        string += 1;
    }
}